<?php

namespace EventPortal\Domain\Component;

use Illuminate\Support\Arr;
use Illuminate\Support\Str;

abstract class Input extends Component
{
    /** @var null|string|array[string] */
    public $value;

    /** @var null|string */
    public $placeholder;

    /** @var bool */
    public $clearable = false;

    /** @var bool */
    public $required = false;

    /** @var array[string] */
    public $validation = [];

    /** @var array[string] */
    public $classes = [];

    /** @var callable */
    public $lookup;

    public function hasOptions(): bool
    {
        return false;
    }

    /**
     * @param string|array $value
     * @return $this
     */
    public function value($value): Input
    {
        $this->value = $value ?: '';

        return $this;
    }

    /**
     * @param null|string|array $rules
     */
    public function rules($rules): self
    {
        if ($rules === null) {
            return $this;
        }

        $rules = $this->parseRules($rules);

        $this->validation = $rules;

        if (in_array('required', $rules, true)) {
            $this->required = true;
        }

        return $this;
    }

    public function placeholder(?string $placeholder): self
    {
        $this->placeholder = $placeholder;

        return $this;
    }

    public function clearable(?bool $clearable): self
    {
        $this->clearable = $clearable ?? false;

        return $this;
    }

    public function isStatus(): bool
    {
        return $this->name === 'status';
    }

    /**
     * @param string|array $rules
     */
    private function parseRules($rules): array
    {
        if (is_string($rules)) {
            $rules = Str::of($rules)->explode('|')->toArray();
        }

        return Arr::wrap($rules);
    }
}
